/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef SENSE_MONITOR2_HH
#define SENSE_MONITOR2_HH

#include "MultiStream.hh"
#include "ranger.hh"
#include "OperStateCondList.hh"
#include "Trend.hh"
#include "MonServer.hh"
#include "autopipe.hh"
#include <fstream>
#include <memory>

class sense_monitor2 : public MultiStream {
public:
   /** Construct a sense_monitor2 monitor class and parse the command-line 
    *  arguments.
    *  \brief SenseMonitor constructor.
    *  \param argc Number of command line arguments
    *  \param argv vector array of command line arguments.
    */
   sense_monitor2(int argc, const char* argv[]);

   /** SenseMonitor destructor.
    */
   ~sense_monitor2(void);

   /** handle any interrupts for the monitor service.
    *  \brief Handle monitor interrupts.
    */
   void Attention(void);

   /** Clear previous results of all range tools.
    */
   void clear(void);

   /** Parse the json configuration file.
    *  \brief parse the configuration.
    *  \param file configuration file path.
    */
   void configure(const std::string& file);

   /** Write all the configuration parameters to the specified output stream
    *  in a human readable format.
    *  \brief Display configuration information.
    *  \param out Output stream to which the configuration will be written.
    */
   void display(std::ostream& out) const;

   /** Write the log-file header to the specified stream.
     */
   void logHeader(const std::string& name, std::ofstream& out, bool app) const;

   /** Write the log-file entry to the specified stream.
     */
   void logEntry(std::ofstream& out, const Time& t0) const;

   /** Perform monitor processing on one stride of data.
    *  \brief Process one stride
    */
   void ProcessData(void);

   /** Write the html status document to the file path specified by mStatusFile.
    *  \brief Write status page.
    *  \param t effective time of writing.
    */
   void put_html(const Time& t) const;

   /** Reset filters and range calculations.
    *  \brief Reset monitor status after shipped data.
    */
   void reset(void);

private:
   typedef std::vector<ranger> ranger_vect;

private:
   std::string       monitorName;
   std::string       mChannel;
   std::string       mOscPath;
   std::string       mOscBreakCond;
   std::string       mOscGateCond;
   std::string       mOscDataStream;
   std::string       mTrendPath;
   std::string       mStatusFile;
   std::string       mCumulativeLog;
   std::string       mJobLogFile;
   double            mSampleRate;
   double            mStride;
   double            minorStride;
   double            mHighPass;            ///< High-pass filter frequency
   double            mHighPassDf;          ///< High-pass filter delta-f
   double            mHighPassAttn;        ///< High-pass filter attenuation
   Interval          mHtmlUpdateInterval;
   std::string       mFilterDesign;
   auto_pipe         mResampler;
   auto_pipe         mConditioner;
   std::unique_ptr<OperStateCondList> mOsc;
   Trend             mTrend;
   ranger_vect       mRangers;
   bool              mEnableServer;
   MonServer         mServer;
   TSeries           mInputSeries;
   bool              mInputStatus;
   size_t            mNStride;
   Time              mStartTime;
   Time              mLastHtmlUpdate;
   std::ofstream     mCumLogStream;
   std::ofstream     mJobLogStream;
};

#endif // !defined(SENSE_MONITOR2_HH)
